/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2011 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * AtomsWithinDistanceOfDescriptor.cpp
 *
 *  Created on: Dec 19, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "AtomsWithinDistanceOfDescriptor.hpp"
#include "AtomsWithinDistanceOfDescriptor_impl.hpp"

#include "Atom/atom.hpp"
#include "LinkedCell/LinkedCell_View.hpp"
#include "CodePatterns/Observer/ObservedContainer_impl.hpp"

using namespace std;


AtomsWithinDistanceOfDescriptor_impl::AtomsWithinDistanceOfDescriptor_impl(const double _distance, const Vector &_position) :
    distance(_distance),
    distanceSquared(distance*distance),
    position(_position)
{}

AtomsWithinDistanceOfDescriptor_impl::~AtomsWithinDistanceOfDescriptor_impl()
{}

bool AtomsWithinDistanceOfDescriptor_impl::predicate(std::pair<atomId_t,atom*> atom)
{
  return ((atom.second->getPosition().DistanceSquared(position) - distanceSquared) <= 0);
}

atom* AtomsWithinDistanceOfDescriptor_impl::find(){
  LinkedCell::LinkedCell_View view = World::getInstance().getLinkedCell(distance);
  LinkedCell::LinkedList list = view.getPointsInsideSphere(distance, position);
  return (list.begin()!=list.end())? static_cast<atom *>(const_cast<TesselPoint *>(*list.begin())):0;
}

std::vector<atom*> AtomsWithinDistanceOfDescriptor_impl::findAll(){
  LinkedCell::LinkedCell_View view = World::getInstance().getLinkedCell(distance);
  LinkedCell::LinkedList list = view.getPointsInsideSphere(distance, position);
  std::vector<atom*> res;
  for (LinkedCell::LinkedList::iterator iter = list.begin(); iter != list.end(); ++iter)
    res.push_back(static_cast<atom *>(const_cast<TesselPoint *>(*iter)));
  return res;
}

AtomDescriptor AtomsWithinDistanceOf(const double distance, const Vector &position)
{
  return AtomDescriptor(AtomDescriptor::impl_ptr(new AtomsWithinDistanceOfDescriptor_impl(distance, position)));
}
